Package: DAMEfinder
Type: Package
Title: Finds DAMEs - Differential Allelicly MEthylated regions
Version: 1.22.0
Authors@R: c(person("Stephany", "Orjuela", role = c("aut", "cre"), 
           email = "sorjuelal@gmail.com", comment = c(ORCID = "0000-0002-1508-461X")),
             person("Dania", "Machlab", role = "aut", email = "dania.machlab@gmail.com"),
             person("Mark", "Robinson", role = "aut", email = "mark.robinson@imls.uzh.ch"))
Description: 'DAMEfinder' offers functionality for taking methtuple or bismark 
        outputs to calculate ASM scores and compute DAMEs. 
        It also offers nice visualization of methyl-circle plots.
License: MIT + file LICENSE
LazyData: FALSE
RoxygenNote: 7.2.2
biocViews: DNAMethylation, DifferentialMethylation, Coverage
Depends: R (>= 4.0)
Imports: stats, Seqinfo, GenomicRanges, IRanges, S4Vectors, readr,
        SummarizedExperiment, GenomicAlignments, stringr, plyr,
        VariantAnnotation, parallel, ggplot2, Rsamtools, BiocGenerics,
        methods, limma, bumphunter, Biostrings, reshape2, cowplot,
        utils
VignetteBuilder: knitr
Suggests: BiocStyle, knitr, rmarkdown, testthat, rtracklayer,
        BSgenome.Hsapiens.UCSC.hg19
BugReports: https://github.com/markrobinsonuzh/DAMEfinder/issues
git_url: https://git.bioconductor.org/packages/DAMEfinder
git_branch: RELEASE_3_22
git_last_commit: a1490db
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.22
Date/Publication: 2025-10-29
NeedsCompilation: no
Packaged: 2025-10-30 03:29:07 UTC; biocbuild
Author: Stephany Orjuela [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-1508-461X>),
  Dania Machlab [aut],
  Mark Robinson [aut]
Maintainer: Stephany Orjuela <sorjuelal@gmail.com>
