# Generated by roxygen2: do not edit by hand

export(calc_asm)
export(calc_derivedasm)
export(dame_track)
export(dame_track_mean)
export(extract_bams)
export(find_dames)
export(get_tstats)
export(methyl_MDS_plot)
export(methyl_circle_plot)
export(methyl_circle_plotCpG)
export(read_tuples)
import(ggplot2)
importFrom(BiocGenerics,"end<-")
importFrom(BiocGenerics,"start<-")
importFrom(BiocGenerics,end)
importFrom(BiocGenerics,start)
importFrom(GenomicAlignments,readGAlignmentPairs)
importFrom(GenomicRanges,GRanges)
importFrom(IRanges,IRanges)
importFrom(S4Vectors,"mcols<-")
importFrom(S4Vectors,mcols)
importFrom(S4Vectors,queryHits)
importFrom(Seqinfo,seqnames)
importFrom(SummarizedExperiment,assay)
importFrom(SummarizedExperiment,assays)
importFrom(SummarizedExperiment,colData)
importFrom(stringr,str_extract)
importFrom(stringr,str_extract_all)
importFrom(stringr,str_locate_all)
